#property link          "https://www.earnforex.com/metatrader-indicators/candlestick-pattern-indicator/"
#property version       "1.02"
#property strict
#property copyright     "EarnForex.com - 2019-2020"
#property description   "This Indicator will detect candlestick patterns" 
#property description   "and send notifications."
#property description   " "
#property description   "WARNING : You use this software at your own risk."
#property description   "The creator of these plugins cannot be held responsible for damage or loss."
#property description   " "
#property icon          "\\Files\\EF-Icon-64x64px.ico"

#include <MQLTA Candlestick Patterns.mqh>

#property indicator_chart_window
#property indicator_buffers 1

#include <MQLTA ErrorHandling.mqh>
#include <MQLTA Utils.mqh>


input string Comment_1="====================";           //Pattern Detector Settings
input string IndicatorName="PTRNDET";                    //Indicator Name (used to draw objects)
input string Comment_2="====================";           //Pattern To Detect
extern int MaxCandle=500;                                //Candles to scan
input bool DetectBullish=true;                           //Detect Bullish Patterns
input bool DetectBearish=true;                           //Detect Bearish Patterns
input bool DetectUncertain=true;                         //Detect Uncertain Patterns
input string Comment_2a="====================";          //Individual Pattern Selection
input bool DetectDojiNeutral=true;                       //Doji
input bool DetectDojiDragonfly=true;                     //Doji Dragonfly
input bool DetectDojiGravestone=true;                    //Doji Gravestone
input bool DetectSpinningTopBull=true;                   //Spinning Top Bullish
input bool DetectSpinningTopBear=true;                   //Spinning Top Bearish
input bool DetectMarubozuUp=true;                        //Marubozu Bullish
input bool DetectMarubozuDown=true;                      //Marubozu Bearish
input bool DetectHammer=true;                            //Hammer
input bool DetectHangingMan=true;                        //Hanging Man
input bool DetectInvertedHammer=true;                    //Inverted Hammer
input bool DetectShootingStar=true;                      //Shooting Star
input bool DetectEngulfingBull=true;                     //Engulfing Bull
input bool DetectEngulfingBear=true;                     //Engulfing Bear
input bool DetectTweezerTop=true;                        //Tweezer Top
input bool DetectTweezerBottom=true;                     //Tweezer Bottom
input bool DetectThreeWhiteSoldier=true;                 //Three White Soldier
input bool DetectThreeBlackCrows=true;                   //Three Black Crows
input bool DetectThreeInsideUp=true;                     //Three Inside Up
input bool DetectThreeInsideDown=true;                   //Three Inside Down
input bool DetectMorningStar=true;                       //Morning Star
input bool DetectEveningStar=true;                       //Evening Star
input string Comment_2b="====================";          //Display Options
input int FontSize=8;                                    //Font Size
input color FontColorBullish=clrGreen;                   //Font Color Bullish Patterns
input color FontColorBearish=clrRed;                     //Font Color Bearish Patterns
input color FontColorUncertain=clrBlack;                 //Font Color Uncertain Patterns
input string Comment_3="====================";           //Notification Options
extern bool EnableNotify=false;                          //Enable Notifications Feature
extern bool SendAlert=true;                              //Send Alert Notification
extern string AlertSound="alert.wav";                    //Alert Sound (wav in the Sounds Folder of MetaTrader)
extern bool SendApp=false;                               //Send Notification to Mobile
extern bool SendEmail=false;                             //Send Notification via Email
input string Comment_4="====================";           //Panel Position
extern int Xoff=20;                                      //Horizontal spacing for the control panel
extern int Yoff=20;                                      //Vertical spacing for the control panel


double BufferPattern[];
bool IsNewCandle=true;

string Font="Verdana";

bool DetectionEnabled=true;
bool PreviousDrawn=false;
bool NotifiedThisCandle=false;

enum ENUM_TYPE_OF_PATTERN{
   UNCERTAIN=0,      //UNCERTAIN
   BULLISH=1,        //BULLISH
   BEARISH=2,        //BEARISH
};


int OnInit(){
   IndicatorSetString(INDICATOR_SHORTNAME,IndicatorName);

   CleanChart();
   ChartSetInteger(0,CHART_EVENT_MOUSE_MOVE,1);
   InitialiseBuffers();
   CreateMiniPanel();
   if(Bars<(MaxCandle+5)){
      MaxCandle=Bars-5;
      Print("Not Enough Historical Candles, Scan Adjusted, Suggested To Load More Candles In The Chart");
      CreateBarsWarning();
   }
   PreviousDrawn=false; 
   DetectionEnabled=true;
   return(INIT_SUCCEEDED);
   
}


int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]){

   if(prev_calculated==0) ArrayInitialize(BufferPattern,0);
   IsNewCandle=IsNewCandleCheck();
   if(IsNewCandle) NotifiedThisCandle=false;
   CleanMiniPanel();
   CreateMiniPanel();
   if(DetectionEnabled) DetectPattern();
   return(rates_total);
}


void OnDeinit(const int reason){
   CleanChart();

}


void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam){
   
   
   if(id==CHARTEVENT_OBJECT_CLICK){
      if(sparam==PanelDraw){
         if(DetectionEnabled) DetectionEnabled=false;
         else DetectionEnabled=true;
         CreateMiniPanel();
      }
      if(sparam==PanelClean){
         CleanPrevious();
      }
   }
   
}


void InitialiseBuffers(){
   IndicatorBuffers(1);
   IndicatorDigits(0);
   SetIndexStyle(0,DRAW_NONE);
   SetIndexBuffer(0,BufferPattern);
   SetIndexLabel(0,"Pattern Detector");

}


datetime NewCandleTime=TimeCurrent();
bool IsNewCandleCheck(){
   if(NewCandleTime==iTime(Symbol(),0,0)) return false;
   else{
      NewCandleTime=iTime(Symbol(),0,0);
      return true;
   }
}


void NotifyPattern(string Pattern){

   if(!SendAlert && !SendApp && !SendEmail) return;
   string EmailSubject=IndicatorName+" "+Symbol()+" Notification ";
   string EmailBody="\r\n"+AccountCompany()+" - "+AccountName()+" - "+IntegerToString(AccountNumber())+"\r\n\r\n"+IndicatorName+" Notification for "+Symbol()+"\r\n\r\n";
   EmailBody+="Detected pattern : "+Pattern+"\r\n\r\n";
   string AlertText=IndicatorName+" - "+Symbol()+" Notification\r\n";
   AlertText+="Detected pattern : "+Pattern+"\r\n";
   string AppText=AccountCompany()+" - "+AccountName()+" - "+IntegerToString(AccountNumber())+" - "+IndicatorName+" - "+Symbol()+" - ";
   AppText+="Detected pattern : "+Pattern+"";
   if(SendAlert) Alert(AlertText);
   if(SendEmail){
      if(!SendMail(EmailSubject,EmailBody)) Print("Error sending email "+IntegerToString(GetLastError()));
   }
   if(SendApp){
      if(!SendNotification(AppText)) Print("Error sending notification "+IntegerToString(GetLastError()));
   }
   NotifiedThisCandle=true;
}


void DetectPattern(){
   
   for(int i=1;i<=MaxCandle;i++){
      if(PreviousDrawn && i>1 && !IsNewCandle) continue;
      if(i==MaxCandle) PreviousDrawn=true;
      if(i>1) NotifiedThisCandle=true;
      string Label="";
      ENUM_TYPE_OF_PATTERN Type;
      BufferPattern[i-1]=0;
      if(DetectMorningStar && IsMorningStar(i)){
         Label="MORNING STAR";
         Type=BULLISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectEveningStar && IsEveningStar(i)){
         Label="EVENING STAR";
         Type=BEARISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectThreeInsideUp && IsThreeInsideUp(i)){
         Label="THREE INSIDE UP";
         Type=BULLISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectThreeInsideDown && IsThreeInsideDown(i)){
         Label="THREE INSIDE DOWN";
         Type=BEARISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectThreeWhiteSoldier && IsThreeWhiteSoldiers(i)){
         Label="THREE WHITE SOLDIER";
         Type=BULLISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectThreeBlackCrows && IsThreeCrows(i)){
         Label="THREE BLACK CROWS";
         Type=BEARISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectEngulfingBull && IsBullishEngulfing(i)){
         Label="BULLISH ENGULFING";
         Type=BULLISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectEngulfingBear && IsBearishEngulfing(i)){
         Label="BEARISH ENGULFIG";
         Type=BEARISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectTweezerTop && IsTweezerTop(i)){
         Label="TWEEZER TOP";
         Type=BEARISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectTweezerBottom && IsTweezerBottom(i)){
         Label="TWEEZER BOTTOM";
         Type=BULLISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectInvertedHammer && IsInvertedHammer(i)){
         Label="INVERTED HAMMER";
         Type=UNCERTAIN;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectShootingStar && IsShootingStar(i)){
         Label="SHOOTING STAR";
         Type=BEARISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectHammer && IsHammer(i)){
         Label="HAMMER";
         Type=BULLISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectHangingMan && IsHangingMan(i)){
         Label="HANGING MAN";
         Type=UNCERTAIN;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectMarubozuUp && IsMarubozuUp(i)){
         Label="MARUBOZU UP";
         Type=BULLISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectMarubozuDown && IsMarubozuDown(i)){
         Label="MARUBOZU DOWN";
         Type=BEARISH;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectSpinningTopBull && IsSpinningTopBullish(i)){
         Label="SPINNING TOP BULLISH";
         Type=UNCERTAIN;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectSpinningTopBear && IsSpinningTopBearish(i)){
         Label="SPINNING TOP BEARISH";
         Type=UNCERTAIN;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectDojiDragonfly && IsDojyDragonfly(i)){
         Label="DOJI DRAGONFLY";
         Type=UNCERTAIN;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectDojiGravestone && IsDojiGravestone(i)){
         Label="DOJI GRAVESTONE";
         Type=UNCERTAIN;
         DrawLabel(i,Label,Type);
         continue;
      }
      if(DetectDojiNeutral && IsDojiNeutral(i)){
         Label="DOJI";
         Type=UNCERTAIN;
         DrawLabel(i,Label,Type);
         continue;
      }
   }
   
}


void DrawLabel(int Index, string Label, ENUM_TYPE_OF_PATTERN Type){
   if(Type==BULLISH && !DetectBullish) return;
   if(Type==BEARISH && !DetectBearish) return;
   if(Type==UNCERTAIN && !DetectUncertain) return;
   string LabelName="";
   string ArrowName="";
   int ArrowType=-1;
   color Color=clrNONE;
   int TextAnchor=0;
   int ArrowAnchor=0;
   datetime CandleTime=Time[Index];
   double PriceLabel=0;
   double PriceArrow=0;
   int ArrowWidth=0;
   if(FontSize>=10) ArrowWidth=2;
   else ArrowWidth=1;
   double PriceOffset=iATR(Symbol(),PERIOD_CURRENT,30,0)/1.5;
   
   if(Type==UNCERTAIN){
      Color=FontColorUncertain;
      TextAnchor=ANCHOR_LEFT;
      PriceLabel=High[Index]+PriceOffset;
      ArrowAnchor=ANCHOR_BOTTOM;
      PriceArrow=High[Index];
      ArrowType=OBJ_ARROW_DOWN;   
      BufferPattern[Index]=0;   
   }
   if(Type==BULLISH){
      Color=FontColorBullish;
      TextAnchor=ANCHOR_RIGHT;
      ArrowAnchor=ANCHOR_TOP;
      PriceLabel=Low[Index]-PriceOffset;
      PriceArrow=Low[Index];
      ArrowType=OBJ_ARROW_UP;
      BufferPattern[Index]=1;   
   }
   if(Type==BEARISH){
      Color=FontColorBearish;
      TextAnchor=ANCHOR_LEFT;
      PriceLabel=High[Index]+PriceOffset;
      ArrowAnchor=ANCHOR_BOTTOM;
      PriceArrow=High[Index];
      ArrowType=OBJ_ARROW_DOWN;
      BufferPattern[Index]=-1;   
   }
   
   LabelName=IndicatorName+"-CANDLE-LBL-"+IntegerToString(CandleTime);
   ArrowName=IndicatorName+"-CANDLE-ARR-"+IntegerToString(CandleTime);
   ObjectCreate(0,LabelName,OBJ_TEXT,0,CandleTime,PriceLabel);
   ObjectSetDouble(0,LabelName,OBJPROP_ANGLE,90);
   ObjectSetInteger(0,LabelName,OBJPROP_ANCHOR,TextAnchor);
   ObjectSetInteger(0,LabelName,OBJPROP_BACK,false);
   ObjectSetInteger(0,LabelName,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,LabelName,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,LabelName,OBJPROP_FONT,Font);
   ObjectSetString(0,LabelName,OBJPROP_TEXT,Label);
   ObjectSetInteger(0,LabelName,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,LabelName,OBJPROP_COLOR,Color);

   ObjectCreate(0,ArrowName,ArrowType,0,CandleTime,PriceArrow);
   ObjectSetInteger(0,ArrowName,OBJPROP_ANCHOR,ArrowAnchor);
   ObjectSetInteger(0,ArrowName,OBJPROP_BACK,false);
   ObjectSetInteger(0,ArrowName,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,ArrowName,OBJPROP_STYLE,STYLE_SOLID);
   ObjectSetInteger(0,ArrowName,OBJPROP_WIDTH,ArrowWidth);
   ObjectSetInteger(0,ArrowName,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,ArrowName,OBJPROP_COLOR,Color);
   
   if(EnableNotify && !NotifiedThisCandle) NotifyPattern(Label);
   
}


void CleanChart(){
   int Window=0;
   for(int i=ObjectsTotal(ChartID(),Window,-1)-1;i>=0;i--){
      if(StringFind(ObjectName(i),IndicatorName,0)>=0){
         ObjectDelete(ObjectName(i));
      }
   }
}


void CleanMiniPanel(){
   int Window=0;
   for(int i=ObjectsTotal(ChartID(),Window,-1)-1;i>=0;i--){
      if(StringFind(ObjectName(i),IndicatorName+"-P-",0)>=0){
         ObjectDelete(ObjectName(i));
      }
   }
}


void CleanPrevious(){
   int Window=0;
   for(int i=ObjectsTotal(ChartID(),Window,-1)-1;i>=0;i--){
      if(StringFind(ObjectName(i),IndicatorName+"-CANDLE-",0)>=0){
         ObjectDelete(ObjectName(i));
      }
   }
   PreviousDrawn=false;
}


string PanelBase=IndicatorName+"-P-BAS";
string PanelLabel=IndicatorName+"-P-LAB";
string PanelDraw=IndicatorName+"-P-DRAW";
string PanelClean=IndicatorName+"-P-CLEAN";

int PanelMovX=26;
int PanelMovY=26;
int PanelLabX=120;
int PanelLabY=PanelMovY;
int PanelRecX=(PanelMovX+2)*2+PanelLabX+2;

void CreateMiniPanel(){
   CleanMiniPanel();
   ObjectCreate(0,PanelBase,OBJ_RECTANGLE_LABEL,0,0,0);
   ObjectSet(PanelBase,OBJPROP_XDISTANCE,Xoff);
   ObjectSet(PanelBase,OBJPROP_YDISTANCE,Yoff);
   ObjectSetInteger(0,PanelBase,OBJPROP_XSIZE,PanelRecX);
   ObjectSetInteger(0,PanelBase,OBJPROP_YSIZE,(PanelMovY+2)*1+2);
   ObjectSetInteger(0,PanelBase,OBJPROP_BGCOLOR,White);
   ObjectSetInteger(0,PanelBase,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,PanelBase,OBJPROP_STATE,false);
   ObjectSetInteger(0,PanelBase,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,PanelBase,OBJPROP_FONTSIZE,8);
   ObjectSet(PanelBase,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,PanelBase,OBJPROP_COLOR,clrBlack);
      
   DrawEdit(PanelLabel,Xoff+2,Yoff+2,PanelLabX,PanelLabY,true,12,"PATTERN DETECTOR",ALIGN_CENTER,"Consolas","DETECTOR",false,clrNavy,clrKhaki,clrBlack); 
   if(DetectionEnabled)  
      DrawEdit(PanelDraw,Xoff+PanelLabX+3,Yoff+2,PanelMovX,PanelMovX,true,12,"Detection Enabled - Click To Pause",ALIGN_CENTER,"Wingdings","J",false,clrNavy,clrKhaki,clrBlack);
   else  
      DrawEdit(PanelDraw,Xoff+PanelLabX+3,Yoff+2,PanelMovX,PanelMovX,true,12,"Detection Disabled - Click To Start",ALIGN_CENTER,"Wingdings","K",false,clrNavy,clrKhaki,clrBlack);
   DrawEdit(PanelClean,Xoff+PanelLabX+PanelMovX+1+3,Yoff+2,PanelMovX,PanelMovX,true,12,"Click To Clean The Chart",ALIGN_CENTER,"Wingdings","I",false,clrNavy,clrKhaki,clrBlack);
   
}

string BarsWarningBase=IndicatorName+"-B-BAS";
string BarsWarningLabel=IndicatorName+"-B-LAB";


void CreateBarsWarning(){
   CleanMiniPanel();
   ObjectCreate(0,BarsWarningBase,OBJ_RECTANGLE_LABEL,0,0,0);
   ObjectSet(BarsWarningBase,OBJPROP_XDISTANCE,Xoff);
   ObjectSet(BarsWarningBase,OBJPROP_YDISTANCE,Yoff+PanelMovY+6);
   ObjectSetInteger(0,BarsWarningBase,OBJPROP_XSIZE,PanelRecX);
   ObjectSetInteger(0,BarsWarningBase,OBJPROP_YSIZE,(PanelMovY+2)*1+2);
   ObjectSetInteger(0,BarsWarningBase,OBJPROP_BGCOLOR,White);
   ObjectSetInteger(0,BarsWarningBase,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,BarsWarningBase,OBJPROP_STATE,false);
   ObjectSetInteger(0,BarsWarningBase,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,BarsWarningBase,OBJPROP_FONTSIZE,8);
   ObjectSet(BarsWarningBase,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,BarsWarningBase,OBJPROP_COLOR,clrBlack);
      
   DrawEdit(BarsWarningLabel,Xoff+2,Yoff+PanelMovY+8,PanelLabX+PanelMovX*2+2,PanelLabY,true,12,"Please load more bars in the chart and reload",ALIGN_CENTER,"Consolas","INSUFFICIENT BARS",false,clrWhite,clrRed,clrBlack); 
   
}

